﻿[CmdletBinding()]
param (
	[Parameter(Mandatory = $false)]
	[System.Management.Automation.PSCredential]
	[System.Management.Automation.Credential()]
	$Credential
)

$tabsList = @(
	[PSCustomObject]@{
		Name       = "ADInfra"
		Label      = "Active Directory & Infra"
		Background = "Aqua"
	}
	[PSCustomObject]@{
		Name       = "ComputerMgmt"
		Label      = "Computer Mgmt Tools"
		Background = "Aquamarine"
	}
	[PSCustomObject]@{
		Name       = "Other"
		Label      = "Other Tools"
		Background = "LightGreen"
	}
)

$toolsList = @(
	[PSCustomObject]@{
		Name       = "DSAC_EXE"
		Label      = "Active Directory Administration Center"
		FilePath   = $env:SystemRoot + "\System32\dsac.exe"
		Parameters = ""
		TabName    = "ADInfra"
	}
	[PSCustomObject]@{
		Name       = "DSA_MSC"
		Label      = "Active Directory Users and Computers"
		FilePath   = $env:SystemRoot + "\System32\dsa.msc"
		Parameters = ""
		TabName    = "ADInfra"
	}
	[PSCustomObject]@{
		Name       = "GPMC_MSC"
		Label      = "Group Policy Management Console"
		FilePath   = $env:SystemRoot + "\System32\gpmc.msc"
		Parameters = ""
		TabName    = "ADInfra"
	}
	[PSCustomObject]@{
		Name       = "DHCP_MSC"
		Label      = "DHCP Management Console"
		FilePath   = $env:SystemRoot + "\System32\dhcpmgmt.msc"
		Parameters = ""
		TabName    = "ADInfra"
	}
	[PSCustomObject]@{
		Name       = "DNSMGMT_MSC"
		Label      = "DNS Management Console"
		FilePath   = $env:SystemRoot + "\System32\dnsmgmt.msc"
		Parameters = ""
		TabName    = "ADInfra"
	}
	[PSCustomObject]@{
		Name       = "REGEDIT_EXE"
		Label      = "Registry Editor"
		FilePath   = $env:SystemRoot + "\regedit.exe"
		Parameters = "/s"
		TabName    = "ComputerMgmt"
	}	
	[PSCustomObject]@{
		Name       = "FSMGMT_MSC"
		Label      = "File Server Management Console"
		FilePath   = $env:SystemRoot + "\System32\fsmgmt.msc"
		Parameters = ""
		TabName    = "ComputerMgmt"
	}
	[PSCustomObject]@{
		Name       = "EVENTVWR_MSC"
		Label      = "Event Viewer"
		FilePath   = $env:SystemRoot + "\System32\eventvwr.msc"
		Parameters = ""
		TabName    = "ComputerMgmt"
	}
	[PSCustomObject]@{
		Name       = "COMPMGMT_MSC"
		Label      = "Computer Management Console"
		FilePath   = $env:SystemRoot + "\system32\compmgmt.msc"
		Parameters = "/s"
		TabName    = "ComputerMgmt"
	}
	[PSCustomObject]@{
		Name       = "CERTLM_MSC"
		Label      = "Certificates - Local computer"
		FilePath   = $env:SystemRoot + "\system32\certlm.msc"
		Parameters = ""
		TabName    = "ComputerMgmt"
	}
	[PSCustomObject]@{
		Name       = "SECPOL_MSC"
		Label      = "Local Security Policy Console"
		FilePath   = $env:SystemRoot + "\system32\secpol.msc"
		Parameters = "/s"
		TabName    = "ComputerMgmt"
	}
	[PSCustomObject]@{
		Name       = "SERVICES_MSC"
		Label      = "Services"
		FilePath   = $env:SystemRoot + "\system32\services.msc"
		Parameters = "/s"
		TabName    = "ComputerMgmt"
	}
	[PSCustomObject]@{
		Name       = "ADV_SYSTEMPROPERTIES"
		Label      = "Advanced System Properties"
		FilePath   = $env:SystemRoot + "\system32\systempropertiesadvanced.exe"
		Parameters = ""
		TabName    = "ComputerMgmt"
	}
	[PSCustomObject]@{
		Name       = "WF_MSC"
		Label      = "Windows Firewall console"
		FilePath   = $env:SystemRoot + "\system32\wf.msc"
		Parameters = ""
		TabName    = "ComputerMgmt"
	}	
	[PSCustomObject]@{
		Name       = "POWERSHELL_EXE"
		Label      = "PowerShell command-line"
		FilePath   = $env:SystemRoot + "\system32\WindowsPowerShell\v1.0\powershell.exe"
		Parameters = ""
		TabName    = "Other"
	}
	[PSCustomObject]@{
		Name       = "POWERSHELL_ISE_EXE"
		Label      = "PowerShell ISE"
		FilePath   = $env:SystemRoot + "\system32\WindowsPowerShell\v1.0\PowerShell_ISE.exe"
		Parameters = ""
		TabName    = "Other"
	}
	[PSCustomObject]@{
		Name       = "NOTEPADPP_EXE"
		Label      = "Notepad ++"
		FilePath   = $env:ProgramFiles + "\Notepad++\notepad++.exe"
		Parameters = ""
		TabName    = "Other"
	}
	[PSCustomObject]@{
		Name       = "CHROME_EXE"
		Label      = "Google Chrome"
		FilePath   = $env:ProgramFiles + "\Google\Chrome\Application\Chrome.exe"
		Parameters = ""
		TabName    = "Other"
	}
)

# check which programs are installed on the computer and remove the non existing ones
foreach ($toolElt in $toolsList) {
	if (-not (Test-Path -Path $toolElt.FilePath -PathType Leaf )) {
		Write-Host "removing " $toolElt.Name
		$toolElt.Name = ""
	}
}

# interface definition

[xml]$xamlInterface = @"
<Window xmlns="http://schemas.microsoft.com/winfx/2006/xaml/presentation"
        xmlns:x="http://schemas.microsoft.com/winfx/2006/xaml" x:Name="MainWindow"
        Title="Administration Tools" Height="500" Width="400"
		SizeToContent="WidthAndHeight"
		ResizeMode = "CanMinimize"
		Background="Aliceblue"
		>
	<Grid Margin="10" >
		<StackPanel Name="Actions" Margin="0,0,0,0">
			<TabControl Name="Tabs" TabStripPlacement="Top">
				<TabItem Name="Default" Header="Default" Visibility="Collapsed">
					<StackPanel />
				</TabItem>
			</TabControl>
			<Button Name="QuitProgram" Content="Quit" Padding="10,5" Margin="5" FontWeight="Bold" />
			<StatusBar Name="StatusBar" HorizontalAlignment="Left" VerticalAlignment="Top" Height="25" IsEnabled="False">
				<TextBlock HorizontalAlignment="Left" VerticalAlignment="Center" FontWeight="Bold" Background="Transparent">
					Running as
				</TextBlock>
				<TextBox Name="RunAsAccount" Text="xxxxxxx@xxxxx" IsReadOnly="True" />
				<TextBlock HorizontalAlignment="Left" VerticalAlignment="Center" FontWeight="Bold" Background="Transparent">
					Local Admin
				</TextBlock>
				<CheckBox Name="IsAdmin" IsChecked="False"/>
			</StatusBar>										   												 
		</StackPanel>
	</Grid>
</Window>
"@

# add the buttons in the XAML definition for existing programs
$xmlActions = $xamlInterface.SelectNodes("//*[@Name='Actions']")
$tabControl = $xmlActions.SelectNodes("//*[@Name='Tabs']")

$isDefaultUsed = $false

# create the tabs
foreach ($tabElt in $tabsList) {
	$tabItem = $tabControl.SelectSingleNode("//*[@Name='{0}']" -f $tabElt.Name)
	if (-not $tabItem) {
		$tabItem = $xamlInterface.CreateNode("element", "TabItem", $xamlInterface.DocumentElement.NamespaceURI)
		$tabItem.SetAttribute("Name", $tabElt.Name)
		$tabItem.SetAttribute("Header", $tabElt.Label)
		$tabItem.SetAttribute("FontWeight","Bold")
		$tabItem.SetAttribute("Visibility", "Visible")
		$tabItem.SetAttribute("Background", $tabElt.Background)
		$tabPanel = $xamlInterface.CreateNode("element", "StackPanel", $xamlInterface.DocumentElement.NamespaceURI)
		$tabPanel.SetAttribute("Name",("{0}_Content" -f $tabElt.Name))
		$tabPanel.SetAttribute("Orientation","Vertical")
		$tabPanel.SetAttribute("HorizontalAlignment","Stretch")
		$tabPanel.SetAttribute("VerticalAlignment","Top")
		$tabPanel.SetAttribute("Background", $tabElt.Background)
		$tabItem.AppendChild($tabPanel)
		$tabControl.AppendChild($tabItem)		
	}
}

foreach ($toolElt in $toolsList) {
	if ($toolElt.Name -ne "") {
		# check if a tab has been defined, otherwise it goes to default
		$tabName = $toolElt.TabName	
		if (-not $tabName) {
			$tabName = "Default"
			$isDefaultUsed = $true
		}
		$tabItem = $tabControl.SelectSingleNode("//*[@Name='{0}']" -f $tabName)
		$tabPanel = $tabItem.SelectSingleNode("//*[@Name='{0}_Content']" -f $tabName)
		$newButton = $xamlInterface.CreateNode("element", "Button", $xamlInterface.DocumentElement.NamespaceURI)
		$newButton.SetAttribute("Name", $toolElt.Name)
		$newButton.SetAttribute("Content", $toolElt.Label)
		$newButton.SetAttribute("Padding", "10,5")
		$newButton.SetAttribute("Margin", "5")
								   
		$newButton.SetAttribute("Background", "White")
		$newButton.SetAttribute("FontWeight","Normal")
		$tabPanel.AppendChild($newButton)
	}															  
}

# if the default tab has been used, display it
if ($isDefaultUsed) {
	$tabItem = $tabControl.SelectSingleNode("//*[@Name='{0}']" -f "Default")
	$tabItem.SetAttribute("Visibility","Visible")
}
else {
	# default is not in use, delete it
	$tabItem = $tabControl.SelectSingleNode("//*[@Name='{0}']" -f "Default")
	$tabControl.RemoveChild($tabItem)
}

# export the xaml form for examination
$xamlInterface.InnerXml | Out-File "C:\Temp\adminui.xml"

[void][System.Reflection.Assembly]::LoadWithPartialName('PresentationFramework')

$okForm = $false
$xamlReader = (New-Object System.Xml.XmlNodeReader $xamlInterface)
try {
	$mainForm = [Windows.Markup.XamlReader]::Load($xamlReader)
	$okForm = $true
}
catch [System.Management.Automation.MethodInvocationException] {
	Write-Warning "Problem occured when loading the XAML code.  Check the syntax for this control..."
	Write-Error $error[0].Exception.Message
}
catch {
	Write-Warning "Unable to load Windows.Markup.XamlReader. Double-check syntax."
	Write-Error $error[0].Exception.Message
}

if (-not $okForm) {
	return
}

$xamlInterface.SelectNodes("//*[@Name]") | ForEach-Object { Set-Variable -Name ($_.Name) -Value $mainForm.FindName($_.Name) }

# add the functions for action when clicking on a button

# Quit button
$QuitProgram.Add_Click(
	{
		Write-Host "Quitting..."
		$mainForm.Close()
	}
)

# other buttons for tools
foreach ($toolElt in $toolsList) {
	if ($toolElt.Name -ne "") {
		$commandLine = 'Start-Process -FilePath "$($toolElt.FilePath)" -WorkingDirectory "$($env:USERPROFILE)"'
		if ($toolElt.Parameters -ne "") {
			$commandLine += ' -ArgumentList "$($toolElt.Parameters)"'
		}
		if ($Credential) {
			$commandLine += ' -Credential $Credential'
		}
		$commandLine = $ExecutionContext.InvokeCommand.ExpandString($commandLine)
		$actionCode = {
			Write-Host "starting $($toolElt.Name)"
			Write-Host "$($toolElt.FilePath) $($toolElt.Parameters)"
			$commandLine
		}
		$actionCode = [Scriptblock]::Create($ExecutionContext.InvokeCommand.ExpandString($actionCode))
		$handlerName = "{0}_OnClick" -f $toolElt.Name
		Set-Variable -Name $handlerName -Value $actionCode
		(Get-Variable -Name $toolElt.Name -ValueOnly).AddHandler([Windows.Controls.Button]::ClickEvent, [Windows.RoutedEventHandler](Get-Variable -Name $handlerName -ValueOnly) )
	}
}

function Test-Administrator {
	$user = [Security.Principal.WindowsIdentity]::GetCurrent()
	(New-Object Security.Principal.WindowsPrincipal $user).IsInRole([Security.Principal.WindowsBuiltinRole]::Administrator)
}

$RunAsAccount.Text = "{0}@{1}" -f $env:USERNAME, $env:USERDNSDOMAIN
$IsAdmin.IsChecked = Test-Administrator
# set the status bar color to the same value as the Window
$StatusBar.Background = $mainForm.Background

$mainForm.ShowDialog()